@@ -19,22 +19,22 @@
 */
 
 #include "module.h"
-#include "module-formats.h"
-#include "signals.h"
-#include "misc.h"
-#include "settings.h"
-#include "levels.h"
-#include "recode.h"
+#include <irssi/src/fe-common/irc/module-formats.h>
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/settings.h>
+#include <irssi/src/core/levels.h>
+#include <irssi/src/core/recode.h>
 
-#include "irc-servers.h"
-#include "irc-channels.h"
-#include "nicklist.h"
-#include "mode-lists.h"
+#include <irssi/src/irc/core/irc-servers.h>
+#include <irssi/src/irc/core/irc-channels.h>
+#include <irssi/src/core/nicklist.h>
+#include <irssi/src/irc/core/mode-lists.h>
 
-#include "../core/module-formats.h"
-#include "printtext.h"
-#include "fe-channels.h"
-#include "fe-irc-server.h"
+#include <irssi/src/fe-common/core/module-formats.h>
+#include <irssi/src/fe-common/core/printtext.h>
+#include <irssi/src/fe-common/core/fe-channels.h>
+#include <irssi/src/fe-common/irc/fe-irc-server.h>
 
 static void print_event_received(IRC_SERVER_REC *server, const char *data,
 				 const char *nick, int target_param);
@@ -511,8 +511,7 @@ static void event_bad_channel_mask(IRC_SERVER_REC *server, const char *data)
 	cannot_join(server, data, IRCTXT_JOINERROR_BAD_MASK);
 }
 
-static void event_477(IRC_SERVER_REC *server, const char *data,
-		      const char *nick)
+static void event_477(IRC_SERVER_REC *server, const char *data, const char *nick)
 {
 	/* Numeric 477 can mean many things:
 	 * modeless channel, cannot join/send to channel (+r/+R/+M).
@@ -531,6 +530,75 @@ static void event_477(IRC_SERVER_REC *server, const char *data,
 	g_free(params);
 }
 
+static void event_489(IRC_SERVER_REC *server, const char *data, const char *nick)
+{
+	/* Numeric 489 can mean one of two things things:
+	 * cannot join to channel (secure only), or not chanop or voice.
+	 * If we tried to join this channel, display the joinerror.
+	 * Otherwise depending on the channel being joined or not
+	 * display the error in the channel or status window.
+	 */
+	IRC_CHANNEL_REC *chanrec;
+	char *params, *channel;
+
+	g_return_if_fail(data != NULL);
+
+	params = event_get_params(data, 2, NULL, &channel);
+
+	chanrec = irc_channel_find(server, channel);
+	if (chanrec != NULL && !chanrec->joined) {
+		cannot_join(server, data, IRCTXT_JOINERROR_SECURE_ONLY);
+	} else {
+		print_event_received(server, data, nick, chanrec == NULL || chanrec->joined);
+	}
+	g_free(params);
+}
+
+static void event_help(IRC_SERVER_REC *server, int formatnum, const char *data)
+{
+	/* Common handling for umerics 704 (RPL_HELPSTART), 705 (RPL_HELPTXT),
+	 * and 706 (RPL_ENDOFHELP); sent as a reply to HELP or HELPOP command.
+	 */
+	char *params, *topic, *help_text;
+
+	g_return_if_fail(data != NULL);
+
+	params = event_get_params(data, 3, NULL, &topic, &help_text);
+
+	g_return_if_fail(help_text != NULL);
+
+	if (help_text[0] == '\0') {
+		/* Empty lines can be used by servers for styling; and we need to replace
+		 * them with something non-empty or they would be dropped when displayed.
+		 */
+		help_text = " ";
+	}
+
+	printformat(server, NULL, MSGLEVEL_CRAP, formatnum, topic, help_text);
+	g_free(params);
+}
+
+static void event_helpstart(IRC_SERVER_REC *server, const char *data, const char *nick)
+{
+	/* Numeric 704 (RPL_HELPSTART) sent as a reply to HELP or HELPOP command.
+	 */
+	event_help(server, IRCTXT_SERVER_HELP_START, data);
+}
+
+static void event_helptxt(IRC_SERVER_REC *server, const char *data, const char *nick)
+{
+	/* Numeric 705 (RPL_HELPTXT), sent as a reply to HELP or HELPOP command.
+	 */
+	event_help(server, IRCTXT_SERVER_HELP_TXT, data);
+}
+
+static void event_endofhelp(IRC_SERVER_REC *server, const char *data, const char *nick)
+{
+	/* Numeric 706 (RPL_ENDOFHELP), sent as a reply to HELP or HELPOP command.
+	 */
+	event_help(server, IRCTXT_SERVER_END_OF_HELP, data);
+}
+
 static void event_target_too_fast(IRC_SERVER_REC *server, const char *data,
 		      const char *nick)
 {
@@ -600,7 +668,7 @@ static void print_event_received(IRC_SERVER_REC *server, const char *data,
 		args = g_strdup(ptr);
 		ptr = strstr(args, " :");
 		if (ptr != NULL)
-			g_memmove(ptr+1, ptr+2, strlen(ptr+1));
+			memmove(ptr+1, ptr+2, strlen(ptr+1));
 	}
 
 	recoded = recode_in(SERVER(server), args, NULL);
@@ -647,6 +715,7 @@ void fe_events_numeric_init(void)
 	last_away_nick = NULL;
 	last_away_msg = NULL;
 
+	/* clang-format off */
 	signal_add("event 221", (SIGNAL_FUNC) event_user_mode);
 	signal_add("event 303", (SIGNAL_FUNC) event_ison);
 	signal_add("event 353", (SIGNAL_FUNC) event_names_list);
@@ -683,11 +752,15 @@ void fe_events_numeric_init(void)
 	signal_add("event 475", (SIGNAL_FUNC) event_bad_channel_key);
 	signal_add("event 476", (SIGNAL_FUNC) event_bad_channel_mask);
 	signal_add("event 477", (SIGNAL_FUNC) event_477);
+	signal_add("event 489", (SIGNAL_FUNC) event_489); /* cannot join to channel (secure only), or not chanop or voice. */
 	signal_add("event 375", (SIGNAL_FUNC) event_motd);
 	signal_add("event 376", (SIGNAL_FUNC) event_motd);
 	signal_add("event 372", (SIGNAL_FUNC) event_motd);
 	signal_add("event 422", (SIGNAL_FUNC) event_motd);
 	signal_add("event 439", (SIGNAL_FUNC) event_target_too_fast);
+	signal_add("event 704", (SIGNAL_FUNC) event_helpstart);
+	signal_add("event 705", (SIGNAL_FUNC) event_helptxt);
+	signal_add("event 706", (SIGNAL_FUNC) event_endofhelp);
 	signal_add("event 707", (SIGNAL_FUNC) event_target_too_fast);
 
         signal_add("default event numeric", (SIGNAL_FUNC) event_numeric);
@@ -699,6 +772,7 @@ void fe_events_numeric_init(void)
 	signal_add("event 004", (SIGNAL_FUNC) event_received);
 	signal_add("event 005", (SIGNAL_FUNC) event_received);
 	signal_add("event 254", (SIGNAL_FUNC) event_received);
+	signal_add("event 354", (SIGNAL_FUNC) event_received);
 	signal_add("event 364", (SIGNAL_FUNC) event_received);
 	signal_add("event 365", (SIGNAL_FUNC) event_received);
 	signal_add("event 381", (SIGNAL_FUNC) event_received);
@@ -726,13 +800,13 @@ void fe_events_numeric_init(void)
 	signal_add("event 478", (SIGNAL_FUNC) event_target_received); /* ban list is full */
 	signal_add("event 482", (SIGNAL_FUNC) event_target_received); /* not chanop */
 	signal_add("event 486", (SIGNAL_FUNC) event_target_received); /* cannot /msg (+R) */
-	signal_add("event 489", (SIGNAL_FUNC) event_target_received); /* not chanop or voice */
 	signal_add("event 494", (SIGNAL_FUNC) event_target_received); /* cannot /msg (own +R) */
 	signal_add("event 506", (SIGNAL_FUNC) event_target_received); /* cannot send (+R) */
 	signal_add("event 716", (SIGNAL_FUNC) event_target_received); /* cannot /msg (+g) */
 	signal_add("event 717", (SIGNAL_FUNC) event_target_received); /* +g notified */
 	signal_add("event 728", (SIGNAL_FUNC) event_target_received); /* quiet (or other) list */
 	signal_add("event 729", (SIGNAL_FUNC) event_target_received); /* end of quiet (or other) list */
+	/* clang-format on */
 }
 
 void fe_events_numeric_deinit(void)
@@ -776,11 +850,15 @@ void fe_events_numeric_deinit(void)
 	signal_remove("event 475", (SIGNAL_FUNC) event_bad_channel_key);
 	signal_remove("event 476", (SIGNAL_FUNC) event_bad_channel_mask);
 	signal_remove("event 477", (SIGNAL_FUNC) event_477);
+	signal_remove("event 489", (SIGNAL_FUNC) event_489);
 	signal_remove("event 375", (SIGNAL_FUNC) event_motd);
 	signal_remove("event 376", (SIGNAL_FUNC) event_motd);
 	signal_remove("event 372", (SIGNAL_FUNC) event_motd);
 	signal_remove("event 422", (SIGNAL_FUNC) event_motd);
 	signal_remove("event 439", (SIGNAL_FUNC) event_target_too_fast);
+	signal_remove("event 704", (SIGNAL_FUNC) event_helpstart);
+	signal_remove("event 705", (SIGNAL_FUNC) event_helptxt);
+	signal_remove("event 706", (SIGNAL_FUNC) event_endofhelp);
 	signal_remove("event 707", (SIGNAL_FUNC) event_target_too_fast);
 
         signal_remove("default event numeric", (SIGNAL_FUNC) event_numeric);
@@ -788,6 +866,7 @@ void fe_events_numeric_deinit(void)
 	signal_remove("event 004", (SIGNAL_FUNC) event_received);
 	signal_remove("event 005", (SIGNAL_FUNC) event_received);
 	signal_remove("event 254", (SIGNAL_FUNC) event_received);
+	signal_remove("event 354", (SIGNAL_FUNC) event_received);
 	signal_remove("event 364", (SIGNAL_FUNC) event_received);
 	signal_remove("event 365", (SIGNAL_FUNC) event_received);
 	signal_remove("event 381", (SIGNAL_FUNC) event_received);
@@ -815,7 +894,6 @@ void fe_events_numeric_deinit(void)
 	signal_remove("event 478", (SIGNAL_FUNC) event_target_received);
 	signal_remove("event 482", (SIGNAL_FUNC) event_target_received);
 	signal_remove("event 486", (SIGNAL_FUNC) event_target_received);
-	signal_remove("event 489", (SIGNAL_FUNC) event_target_received);
 	signal_remove("event 494", (SIGNAL_FUNC) event_target_received);
 	signal_remove("event 506", (SIGNAL_FUNC) event_target_received);
 	signal_remove("event 716", (SIGNAL_FUNC) event_target_received);
