@@ -19,19 +19,21 @@
 */
 
 #include "module.h"
-#include "modules.h"
-#include "network.h"
-#include "net-sendbuffer.h"
-#include "rawlog.h"
-#include "misc.h"
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/modules.h>
+#include <irssi/src/core/net-sendbuffer.h>
+#include <irssi/src/core/network.h>
+#include <irssi/src/core/rawlog.h>
+#include <irssi/src/core/refstrings.h>
 
-#include "irc-servers.h"
-#include "irc-channels.h"
-#include "servers-redirect.h"
+#include <irssi/src/irc/core/irc-channels.h>
+#include <irssi/src/irc/core/irc-servers.h>
+#include <irssi/src/irc/core/servers-redirect.h>
 
 char *current_server_event;
 static int signal_default_event;
 static int signal_server_event;
+static int signal_server_event_tags;
 static int signal_server_incoming;
 
 #ifdef BLOCKING_SOCKETS
@@ -45,11 +47,12 @@ static void strip_params_colon(char *const);
 /* The core of the irc_send_cmd* functions. If `raw' is TRUE, the `cmd'
    won't be checked at all if it's 512 bytes or not, or if it contains
    line feeds or not. Use with extreme caution! */
-void irc_send_cmd_full(IRC_SERVER_REC *server, const char *cmd,
-		       int send_now, int immediate, int raw)
+void irc_send_cmd_full(IRC_SERVER_REC *server, const char *cmd, int irc_send_when, int raw)
 {
-	char str[513];
+	GString *str;
 	int len;
+	guint pos;
+	gboolean server_supports_tag;
 
 	g_return_if_fail(server != NULL);
 	g_return_if_fail(cmd != NULL);
@@ -57,65 +60,97 @@ void irc_send_cmd_full(IRC_SERVER_REC *server, const char *cmd,
 	if (server->connection_lost)
 		return;
 
-	len = strlen(cmd);
+	str = g_string_sized_new(MAX_IRC_USER_TAGS_LEN + 2 /* `@'+SPACE */ +
+				 server->max_message_len + 2 /* CR+LF */ + 1 /* `\0' */);
+
 	if (server->cmdcount == 0)
 		irc_servers_start_cmd_timeout();
 	server->cmdcount++;
 
+	pos = g_slist_length(server->cmdqueue);
+	if (server->cmdlater > pos / 2) {
+		server->cmdlater = pos / 2;
+		pos = 0;
+	} else {
+		pos -= 2 * server->cmdlater;
+	}
+
 	if (!raw) {
+		const char *tmp = cmd;
+
+		server_supports_tag = server->cap_supported != NULL &&
+			g_hash_table_lookup_extended(server->cap_supported, CAP_MESSAGE_TAGS, NULL, NULL);
+
+		if (*cmd == '@' && server_supports_tag) {
+			const char *end;
+
+			while (*tmp != ' ' && *tmp != '\0')
+				tmp++;
+
+			end = tmp;
+
+			if (tmp - cmd > MAX_IRC_USER_TAGS_LEN) {
+				g_warning("irc_send_cmd_full(); tags too long(%ld)", tmp - cmd);
+				while (tmp - cmd > MAX_IRC_USER_TAGS_LEN && cmd != tmp - 1) tmp--;
+				while (*tmp != ',' && cmd != tmp - 1) tmp--;
+			}
+			if (cmd != tmp)
+				g_string_append_len(str, cmd, tmp - cmd);
+
+			tmp = end;
+			while (*tmp == ' ') tmp++;
+
+			if (*tmp != '\0' && str->len > 0)
+				g_string_append_c(str, ' ');
+		}
+		len = strlen(tmp);
+
 		/* check that we don't send any longer commands
 		   than 510 bytes (2 bytes for CR+LF) */
-		strncpy(str, cmd, 510);
-		if (len > 510) len = 510;
-		str[len] = '\0';
-                cmd = str;
-	}
-
-	if (send_now) {
-		rawlog_output(server->rawlog, cmd);
-		server_redirect_command(server, cmd, server->redirect_next);
-                server->redirect_next = NULL;
+		g_string_append_len(str, tmp, len > server->max_message_len ?
+				    server->max_message_len : len);
+	} else {
+		g_string_append(str, cmd);
 	}
 
 	if (!raw) {
-                /* Add CR+LF to command */
-		str[len++] = 13;
-		str[len++] = 10;
-		str[len] = '\0';
+		/* Add CR+LF to command */
+		g_string_append(str, "\r\n");
 	}
 
-	if (send_now) {
-                irc_server_send_data(server, cmd, len);
-	} else {
-
+	if (irc_send_when == IRC_SEND_NOW) {
+		irc_server_send_and_redirect(server, str, server->redirect_next);
+		g_string_free(str, TRUE);
+	} else if (irc_send_when == IRC_SEND_NEXT) {
 		/* add to queue */
-		if (immediate) {
-			server->cmdqueue = g_slist_prepend(server->cmdqueue,
-							   server->redirect_next);
-			server->cmdqueue = g_slist_prepend(server->cmdqueue,
-							   g_strdup(cmd));
-		} else {
-			server->cmdqueue = g_slist_append(server->cmdqueue,
-							  g_strdup(cmd));
-			server->cmdqueue = g_slist_append(server->cmdqueue,
-							  server->redirect_next);
-		}
+		server->cmdqueue = g_slist_prepend(server->cmdqueue, server->redirect_next);
+		server->cmdqueue = g_slist_prepend(server->cmdqueue, g_string_free(str, FALSE));
+	} else if (irc_send_when == IRC_SEND_NORMAL) {
+		server->cmdqueue = g_slist_insert(server->cmdqueue, server->redirect_next, pos);
+		server->cmdqueue = g_slist_insert(server->cmdqueue, g_string_free(str, FALSE), pos);
+	} else if (irc_send_when == IRC_SEND_LATER) {
+		server->cmdqueue = g_slist_append(server->cmdqueue, g_string_free(str, FALSE));
+		server->cmdqueue = g_slist_append(server->cmdqueue, server->redirect_next);
+		server->cmdlater++;
+	} else {
+		g_warn_if_reached();
 	}
-        server->redirect_next = NULL;
+
+	server->redirect_next = NULL;
 }
 
 /* Send command to IRC server */
 void irc_send_cmd(IRC_SERVER_REC *server, const char *cmd)
 {
-	GTimeVal now;
+	gint64 now;
 	int send_now;
 
-        g_get_current_time(&now);
-	send_now = g_timeval_cmp(&now, &server->wait_cmd) >= 0 &&
-		(server->cmdcount < server->max_cmds_at_once ||
-		 server->cmd_queue_speed <= 0);
+	now = g_get_real_time();
+	send_now = now >= server->wait_cmd &&
+	           (server->cmdcount < server->max_cmds_at_once ||
+		    server->cmd_queue_speed <= 0);
 
-        irc_send_cmd_full(server, cmd, send_now, FALSE, FALSE);
+	irc_send_cmd_full(server, cmd, send_now ? IRC_SEND_NOW : IRC_SEND_NORMAL, FALSE);
 }
 
 /* Send command to IRC server */
@@ -139,7 +174,7 @@ void irc_send_cmd_now(IRC_SERVER_REC *server, const char *cmd)
 {
 	g_return_if_fail(cmd != NULL);
 
-        irc_send_cmd_full(server, cmd, TRUE, TRUE, FALSE);
+	irc_send_cmd_full(server, cmd, IRC_SEND_NOW, FALSE);
 }
 
 /* Send command to server putting it at the beginning of the queue of
@@ -149,7 +184,15 @@ void irc_send_cmd_first(IRC_SERVER_REC *server, const char *cmd)
 {
 	g_return_if_fail(cmd != NULL);
 
-        irc_send_cmd_full(server, cmd, FALSE, TRUE, FALSE);
+	irc_send_cmd_full(server, cmd, IRC_SEND_NEXT, FALSE);
+}
+
+/* Send command to server putting it at the end of the queue. */
+void irc_send_cmd_later(IRC_SERVER_REC *server, const char *cmd)
+{
+	g_return_if_fail(cmd != NULL);
+
+	irc_send_cmd_full(server, cmd, IRC_SEND_LATER, FALSE);
 }
 
 static char *split_nicks(const char *cmd, char **pre, char **nicks, char **post, int arg)
@@ -158,6 +201,14 @@ static char *split_nicks(const char *cmd, char **pre, char **nicks, char **post,
 
 	*pre = g_strdup(cmd);
 	*post = *nicks = NULL;
+
+	if (**pre == '@') {
+		/* the message-tags "add" one space separated argument
+		   in front of the non message-tagged IRC commands. So
+		   the nicks are now off-set by one to the right. */
+		arg++;
+	}
+
 	for (p = *pre; *p != '\0'; p++) {
 		if (*p != ' ')
 			continue;
@@ -342,13 +393,107 @@ static void irc_server_event(IRC_SERVER_REC *server, const char *line,
 	g_free(event);
 }
 
-static char *irc_parse_prefix(char *line, char **nick, char **address)
+static void unescape_tag(char *tag)
+{
+	char *tmp;
+
+	if (tag == NULL)
+		return;
+
+	tmp = tag;
+	for (; *tmp != '\0'; tmp++, tag++) {
+		if (*tmp == '\\') {
+			tmp++;
+			if (*tmp == '\0')
+				break;
+			switch (*tmp) {
+			case ':':
+				*tag = ';';
+				break;
+			case 'n':
+				*tag = '\n';
+				break;
+			case 'r':
+				*tag = '\r';
+				break;
+			case 's':
+				*tag = ' ';
+				break;
+			default:
+				*tag = *tmp;
+				break;
+			}
+		} else {
+			*tag = *tmp;
+		}
+	}
+	*tag = '\0';
+}
+
+static gboolean i_str0_equal(const char *s1, const char *s2)
+{
+	return g_strcmp0(s1, s2) == 0;
+}
+
+GHashTable *irc_parse_message_tags(const char *tags)
+{
+	char **split, **tmp, **kv;
+	GHashTable *hash;
+
+	hash = g_hash_table_new_full(g_str_hash, (GEqualFunc) i_str0_equal,
+	                             (GDestroyNotify) i_refstr_release, (GDestroyNotify) g_free);
+	split = g_strsplit(tags, ";", -1);
+	for (tmp = split; *tmp != NULL; tmp++) {
+		if (*tmp[0] == '\0')
+			continue;
+		kv = g_strsplit(*tmp, "=", 2);
+		unescape_tag(kv[1]);
+		g_hash_table_replace(hash, i_refstr_intern(kv[0]),
+		                     g_strdup(kv[1] == NULL ? "" : kv[1]));
+		g_strfreev(kv);
+	}
+	g_strfreev(split);
+	return hash;
+}
+
+static void irc_server_event_tags(IRC_SERVER_REC *server, const char *line, const char *nick,
+                                  const char *address, const char *tags)
+{
+	char *timestr;
+	GHashTable *tags_hash = NULL;
+
+	if (tags != NULL && *tags != '\0') {
+		tags_hash = irc_parse_message_tags(tags);
+		if ((timestr = g_hash_table_lookup(tags_hash, "time")) != NULL) {
+			server_meta_stash(SERVER(server), "time", timestr);
+		}
+	}
+
+	if (*line != '\0')
+		signal_emit_id(signal_server_event, 4, server, line, nick, address);
+
+	if (tags_hash != NULL)
+		g_hash_table_destroy(tags_hash);
+}
+
+static char *irc_parse_prefix(char *line, char **nick, char **address, char **tags)
 {
 	char *p;
 
-	*nick = *address = NULL;
+	*nick = *address = *tags = NULL;
 
-	/* :<nick> [["!" <user>] "@" <host>] SPACE */
+	/* ["@" <tags> SPACE] :<nick> [["!" <user>] "@" <host>] SPACE */
+
+	if (*line == '@') {
+		*tags = ++line;
+		while (*line != '\0' && *line != ' ') {
+			line++;
+		}
+		if (*line == ' ') {
+			*line++ = '\0';
+			while (*line == ' ') line++;
+		}
+	}
 
 	if (*line != ':')
 		return line;
@@ -382,14 +527,16 @@ static char *irc_parse_prefix(char *line, char **nick, char **address)
 /* Parse command line sent by server */
 static void irc_parse_incoming_line(IRC_SERVER_REC *server, char *line)
 {
-	char *nick, *address;
+	char *nick, *address, *tags;
 
 	g_return_if_fail(server != NULL);
 	g_return_if_fail(line != NULL);
 
-	line = irc_parse_prefix(line, &nick, &address);
-	if (*line != '\0')
-		signal_emit_id(signal_server_event, 4, server, line, nick, address);
+	line = irc_parse_prefix(line, &nick, &address, &tags);
+	if (*line != '\0' || tags != NULL)
+		signal_emit_id(signal_server_event_tags, 5, server, line, nick, address, tags);
+
+	server_meta_clear_all(SERVER(server));
 }
 
 /* input function: handle incoming server messages */
@@ -432,27 +579,30 @@ static void irc_init_server(IRC_SERVER_REC *server)
 	if (!IS_IRC_SERVER(server))
 		return;
 
-	server->readtag =
-		g_input_add(net_sendbuffer_handle(server->handle),
-			    G_INPUT_READ,
-			    (GInputFunction) irc_parse_incoming, server);
+	server->readtag = i_input_add(net_sendbuffer_handle(server->handle), I_INPUT_READ,
+	                              (GInputFunction) irc_parse_incoming, server);
 }
 
 void irc_irc_init(void)
 {
 	signal_add("server event", (SIGNAL_FUNC) irc_server_event);
+	signal_add("server event tags", (SIGNAL_FUNC) irc_server_event_tags);
 	signal_add("server connected", (SIGNAL_FUNC) irc_init_server);
+	signal_add("server connection switched", (SIGNAL_FUNC) irc_init_server);
 	signal_add("server incoming", (SIGNAL_FUNC) irc_parse_incoming_line);
 
 	current_server_event = NULL;
 	signal_default_event = signal_get_uniq_id("default event");
 	signal_server_event = signal_get_uniq_id("server event");
+	signal_server_event_tags = signal_get_uniq_id("server event tags");
 	signal_server_incoming = signal_get_uniq_id("server incoming");
 }
 
 void irc_irc_deinit(void)
 {
 	signal_remove("server event", (SIGNAL_FUNC) irc_server_event);
+	signal_remove("server event tags", (SIGNAL_FUNC) irc_server_event_tags);
 	signal_remove("server connected", (SIGNAL_FUNC) irc_init_server);
+	signal_remove("server connection switched", (SIGNAL_FUNC) irc_init_server);
 	signal_remove("server incoming", (SIGNAL_FUNC) irc_parse_incoming_line);
 }
